@@ -68,6 +68,8 @@ static void chat_window_conv_activate_cb      (GtkWidget             *menuitem,
 					       GossipChatWindow      *window);
 static void chat_window_close_activate_cb     (GtkWidget	     *menuitem,
 					       GossipChatWindow      *window);
+static void chat_window_cut_activate_cb       (GtkWidget             *menuitem,
+					       GossipChatWindow      *window);
 static void chat_window_copy_activate_cb      (GtkWidget             *menuitem,
 					       GossipChatWindow      *window);
 static void chat_window_paste_activate_cb     (GtkWidget             *menuitem,
@@ -149,6 +151,7 @@ struct _GossipChatWindowPriv {
 	GtkWidget   *m_conv_log;
 	GtkWidget   *m_conv_info;
 	GtkWidget   *m_conv_close;
+	GtkWidget   *m_edit_cut;
 	GtkWidget   *m_edit_copy;
 	GtkWidget   *m_edit_paste;
 	GtkWidget   *m_tabs_next;
@@ -233,6 +236,7 @@ gossip_chat_window_init (GossipChatWindow *window)
 				       "menu_conv_info", &priv->m_conv_info,
 				       "menu_conv_log", &priv->m_conv_log,
 				       "menu_conv_close", &priv->m_conv_close,
+				       "menu_edit_cut", &priv->m_edit_cut,
 				       "menu_edit_copy", &priv->m_edit_copy,
 				       "menu_edit_paste", &priv->m_edit_paste,
 				       "menu_tabs_next", &priv->m_tabs_next,
@@ -279,6 +283,10 @@ gossip_chat_window_init (GossipChatWindow *window)
 			  "activate",
 			  G_CALLBACK (chat_window_close_activate_cb),
 			  window);
+	g_signal_connect (priv->m_edit_cut,
+			  "activate",
+			  G_CALLBACK (chat_window_cut_activate_cb),
+			  window);
 	g_signal_connect (priv->m_edit_copy,
 			  "activate",
 			  G_CALLBACK (chat_window_copy_activate_cb),
@@ -389,17 +397,12 @@ chat_window_get_status_pixbuf (GossipChatWindow *window,
 		pixbuf = gossip_utils_get_pixbuf_offline ();
 	}
 	else {
-		GossipContact *contact;
-
+		GossipContact  *contact;
+		GossipPresence *presence;
+		
 		contact = gossip_chat_get_contact (chat);
-		if (gossip_contact_is_online (contact)) {
-			GossipPresence *presence;
-
-			presence = gossip_contact_get_presence (contact);
-			pixbuf = gossip_utils_get_pixbuf_from_presence (presence);
-		} else {
-			pixbuf = gossip_utils_get_pixbuf_offline ();
-		}
+		presence = gossip_contact_get_presence (contact);
+		pixbuf = gossip_presence_get_pixbuf (presence);
 	}
 
 	return pixbuf;
@@ -443,8 +446,6 @@ chat_window_has_toplevel_focus (GossipChatWindow *window)
 	return focus;
 }
 
-#define TAB_MIN_WIDTH 130
-
 static GtkWidget *
 chat_window_create_label (GossipChatWindow *window,
 			  GossipChat       *chat)
@@ -480,9 +481,6 @@ chat_window_create_label (GossipChatWindow *window,
 
 	chat_window_update_tooltip (window, chat);
 	
-	/* Set minimum size. */
-	gtk_widget_set_size_request (hbox, TAB_MIN_WIDTH, -1);
-	
 	gtk_misc_set_padding (GTK_MISC (name_label), 2, 0);
 	gtk_misc_set_alignment (GTK_MISC (name_label), 0.0, 0.5);
 
@@ -631,6 +629,19 @@ chat_window_close_activate_cb (GtkWidget        *menuitem,
 	gossip_chat_window_remove_chat (window, window->priv->current_chat);
 }
 
+static void
+chat_window_cut_activate_cb (GtkWidget        *menuitem,
+			      GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+	
+	g_return_if_fail (GOSSIP_IS_CHAT_WINDOW (window));
+
+	priv = window->priv;
+
+	gossip_chat_cut (priv->current_chat);
+}
+
 static void
 chat_window_copy_activate_cb (GtkWidget        *menuitem,
 			      GossipChatWindow *window)
